from typing import Dict
import ctypes
from ..native.Python.QuixSdkStreaming.Models.StreamWriter.EventDataBuilder import EventDataBuilder as edbi
from ..helpers.nativedecorator import nativedecorator


@nativedecorator
class EventDataBuilder(object):
    """
        Builder for creating event data packages for StreamPropertiesWriter
    """

    def __init__(self, net_pointer: ctypes.c_void_p):
        """
            Initializes a new instance of EventDataBuilder.

            Parameters:

            net_pointer: Pointer to an instance of a .net EventDataBuilder.
        """

        if net_pointer is None:
            raise Exception("EventDataBuilder is none")

        self.__interop = edbi(net_pointer)
        self.__entered = False

    def __enter__(self):
        self.__entered = True

    def add_value(self, event_id: str, value: str) -> 'EventDataBuilder':
        """
        Adds new event at the time the builder is created for
        :param event_id: The id of the event to set the value for
        :param value: the string value
        :return: The builder
        """

        new = edbi(self.__interop.AddValue(event_id, value))
        if new != self.__interop:
            self.__interop._dispose_ptr()
            self.__interop = new
        return self

    def add_tag(self, tag_id: str, value: str) -> 'EventDataBuilder':
        """
        Sets tag value for the values
        :param tag_id: The id of the tag
        :param value: The value of the tag
        :return: The builder
        """

        new = edbi(self.__interop.AddTag(tag_id, value))
        if new != self.__interop:
            self.__interop._dispose_ptr()
            self.__interop = new
        return self

    def add_tags(self, tags: Dict[str, str]) -> 'EventDataBuilder':
        """
            Copies the tags from the specified dictionary. Conflicting tags will be overwritten
            :param tags: The tags to add

        :return: EventDataBuilder
        """

        if tags is None:
            return self

        # TODO
        prep_tags_dict = System.Collections.Generic.List[System.Collections.Generic.KeyValuePair[str, str]]([])
        for key, val in tags.items():
            prep_tags_dict.Add(System.Collections.Generic.KeyValuePair[str, str](key, val))
        self.__wrapped.AddTags(prep_tags_dict)
        return self

    def write(self):
        """
        Writes the values to the StreamEventsWriter buffer. See StreamEventsWriter buffer settings for more information when the values are sent to the broker
        """

        self.__interop.Write()

        if not self.__entered:
            self.dispose()
